// 002 u_c_calcul_charges_sarl
// 09 jan 2020

// -- (C) John COLIBRI 2020
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_calcul_charges_sarl;
  interface
    uses Classes, u_c_basic_object
        , u_strings, u_stringlist, u_c_stringlist
        ;

    type c_calcul_charges_sarl=
             class(c_stringlist)
               m_title: String;
               m_month_count: Integer;

               m_c_key_list, m_c_value_list: tStringList;

               Constructor create_calcul_charges_sarl(p_name, p_path, p_file_name: String);

               function f_amount(p_key: String): Double;
               function f_rate(p_key: String): Double;
               procedure load_parameters;
               procedure display_parameters;
               procedure compute_charges(p_salaire, p_cotisations_sociales_obligatoire_precedentes: Double);

               Destructor Destroy; Override;
             end; // c_calcul_charges_sarl

  implementation
    uses SysUtils, Math, u_c_display;

    // -- c_calcul_charges_sarl

    Constructor c_calcul_charges_sarl.create_calcul_charges_sarl(p_name, p_path, p_file_name: String);
      begin
        Inherited create_stringlist(p_name, p_path, p_file_name);
        m_c_key_list:= tStringList.Create;
        m_c_value_list:= tStringList.Create;
      end; // create_calcul_charges_sarl

    procedure c_calcul_charges_sarl.load_parameters;

      procedure _add_key_value(p_key, p_value: String);
        begin
          display('add '+ p_key+ ' '+ p_value);
          m_c_key_list.Add(p_key);
          m_c_value_list.Add(p_value);
        end; // _add_key_value

      var l_key, l_value: String;

      begin // load_parameters
        stop_if_empty;
        display(m_c_stringlist.Text);

        while not f_end_of_file do
        begin
          if not f_is_blank_or_comment
            then begin
                // _add_key_value(f_next_value, f_next_value);
                l_key:= f_next_value; l_value:= f_next_value;
                _add_key_value(l_key, l_value);
              end
            else
              if m_title= ''
                then m_title:= m_trimmed_line
                else _add_key_value('','');

          read_line;
        end; // while not f_end_of_file
        display(m_c_key_list.Text);
        display(m_c_value_list.Text);
      end; // load_parameters

    function c_calcul_charges_sarl.f_amount(p_key: String): Double;
      var l_index_of: integer;
      begin
        l_index_of:= m_c_key_list.IndexOf(p_key);
        if l_index_of< 0
          then begin
              display_bug_stop('no_key '+ p_key);
              Result:= 0;
            end
          else Result:= StrToFloat(m_c_value_list[l_index_of]);
      end; // f_amount

    function c_calcul_charges_sarl.f_rate(p_key: String): Double;
      var l_index_of: integer;
      begin
        l_index_of:= m_c_key_list.IndexOf(p_key);
        if l_index_of< 0
          then begin
              display_bug_stop('no_key '+ p_key);
              Result:= 0;
            end
          else Result:= StrToFloat(m_c_value_list[l_index_of]);
      end; // f_rate

    procedure c_calcul_charges_sarl.display_parameters;
      var l_sarl_parameter_index: Integer;
          l_key: String;
      begin
        display_line;
        display(m_title);

        with m_c_key_list do
          for l_sarl_parameter_index:= 0 to Count- 1 do
          begin
            l_key:= Strings[l_sarl_parameter_index];
            if l_key= ''
              then display_line
              else display(Format('%-25s %10.2f', [l_key, f_amount(l_key)]));
          end;
      end; // display_parameters

    procedure c_calcul_charges_sarl.compute_charges(p_salaire, p_cotisations_sociales_obligatoire_precedentes: Double);
      var l_total: Double;

      procedure _display(p_name, p_threshold: String;
          p_base, p_rate, p_value, p_category_total: Double);
        var l_category_total: String;
        begin
          p_value:= p_value/ m_month_count;
          // p_category_total:= p_category_total/ m_month_count;

          if p_category_total= 0
            then l_category_total:= ''
            else l_category_total:= Format('  %10.2f', [p_category_total]);

          l_total:= l_total+ p_value;
          display(Format('%-31s %-14s %10.2f %5.2f %10.2f %s ',
              [p_name, p_threshold, p_base, p_rate* 100, p_value, l_category_total]));
        end; // _display

      procedure _compute_maladie;
        var l_value, l_rate: Double;
        begin
(*
          display(Format('%10.2f %10.f %10.2f',
               [p_salaire, 0.40* f_amount('plafond_secu')/100, 1.10* f_amount('plafond_secu')/100]));
*)
          if p_salaire< 0.40* f_amount('plafond_secu')
            then begin
                l_rate:= (f_rate('maladie_T1')- f_rate('maladie_T2'))/ (1.10* f_amount('plafond_secu'))* p_salaire
                    + (f_rate('maladie_T2')- f_rate('maladie_T3'))/ (0.40* f_amount('plafond_secu'))* p_salaire
                    + f_rate('maladie_T3');
                l_rate:= l_rate/ 100;
                l_value:= p_salaire* l_rate;
                _display('maladie_1', '< 0.4* pss', p_salaire, l_rate, l_value, 0);
              end
            else
              if p_salaire< 1.10* f_amount('plafond_secu')
                then begin
(*
                    display(Format('%10.2f %10.2f ',
                        [(f_rate('maladie_T1')- f_rate('maladie_T2'))/ (1.10* f_amount('plafond_secu'))* p_salaire,
                         f_rate('maladie_T2')]));
*)
                    l_rate:= (f_rate('maladie_T1')- f_rate('maladie_T2'))/ (1.10* f_amount('plafond_secu'))* p_salaire
                        + f_rate('maladie_T2') ;
                    l_rate:= l_rate/ 100;
                    l_value:= p_salaire* l_rate;
                    _display('maladie_1', '< 1.1* pss', p_salaire, l_rate, l_value, 0);
                  end
                else begin
                    l_rate:= f_rate('maladie_T1');
                    l_rate:= l_rate/ 100;
                    l_value:= p_salaire* l_rate;
                    _display('maladie_1', '> 1.1* pss', p_salaire, l_rate, l_value, 0);
                  end;
        end; // _compute_maladie

      procedure _compute_allocations;
        var l_value, l_rate: Double;
        begin
          if p_salaire< 1.10* f_amount('plafond_secu')
            then begin
                display('no_alloc');
              end
            else
              if (p_salaire>= 1.10* f_amount('plafond_secu')) and (p_salaire< 1.40* f_amount('plafond_secu'))
                then begin
                    l_rate:= f_rate('allocations_T1')/ (0.30* f_amount('plafond_secu'))*
                        (p_salaire- 1.10*  f_amount('plafond_secu')) ;
                    l_rate:= l_rate/ 100;
                    l_value:= p_salaire* l_rate;
                    _display('alloc_2', '1.1 1.4 pss', p_salaire, l_rate, l_value, 0);
                  end
                else
                  if (p_salaire>= 1.40* f_amount('plafond_secu'))
                    then begin
                        l_rate:= f_rate('allocations_T1')/ 100;
                        l_value:= p_salaire* l_rate;
                        _display('alloc_3', '> 1.4 pss', p_salaire, l_rate, l_value, 0);
                      end;
        end; // _compute_allocations

      procedure _compute_retraite_base;
        var l_base, l_rate, l_value: Double;
        begin
          l_base:= Min(p_salaire, f_amount('plafond_secu'));
          l_rate:= f_rate('retraite_base_1')/ 100;
          l_value:= l_base* l_rate;
          _display('retraite_1', '< pss', l_base, l_rate, l_value, 0);

          l_base:= p_salaire- f_amount('plafond_secu');
          if l_base> 0
            then begin
                l_rate:= f_rate('retraite_base_2')/ 100;
                l_value:= l_base* l_rate;
                _display('retraite_2', '> pss', l_base, l_rate, l_value, 0);
              end;
        end; // _compute_retraite_base

      procedure _compute_retraite_complementaire;
        var l_base, l_rate, l_value: Double;
        begin
          l_base:= Min(p_salaire, f_amount('plafond_complementaire'));
          l_rate:= f_rate('retraite_complementaire_1')/ 100;
          l_value:= l_base* l_rate;
          _display('complem_1', '< compl', l_base, l_rate, l_value, 0);

          l_base:= p_salaire- f_amount('plafond_complementaire');
          if l_base> 0
            then begin
                l_base:= Min(l_base, 4* f_amount('plafond_secu'));
                l_rate:= f_rate('retraite_complementaire_2')/ 100;
                l_value:= l_base* l_rate;
                _display('complem_2', '> compl', l_base, l_rate, l_value, 0);
              end
        end; // _compute_retraite_complementaire

      procedure _compute_formation_professionnelle;
        var l_base, l_rate, l_value: Double;
        begin
          l_base:= f_amount('plafond_secu');

          l_rate:= f_rate('formation_professionnelle')/ 100;
          l_value:= l_base* l_rate;
          _display('form_profess', 'plaf', l_base, l_rate, l_value, 0);
        end; // _compute_formation_professionnelle

      procedure _compute_csg_crds;
        var l_base, l_rate, l_value: Double;
        begin
          l_base:= p_salaire+ p_cotisations_sociales_obligatoire_precedentes;
          l_rate:= f_rate('csg_crds_non_deductible')/ 100;
          l_value:= l_base* l_rate;
          _display('csg_crds', 'sal+cot_p', l_base, l_rate, l_value, 0);

          l_rate:= f_rate('csg_deductible')/ 100;
          l_value:= l_base* l_rate;
          _display('csg_deduc', 'sal+cot_p', l_base, l_rate, l_value, 0);
        end; // _compute_csg_crds

      procedure _compute_xxx;
        begin
        end; // _compute_xxx

      begin // compute_charges
        display_line;
        display('-----');
        display(Format('Charges Salaire: %10.2f  cotisations n- 1 %10.2f',
            [p_salaire, p_cotisations_sociales_obligatoire_precedentes]));
        display_line;

        l_total:= 0;

        _compute_maladie;

        display_line;
        _compute_allocations;
        display_line;
        _compute_retraite_base;
        _compute_retraite_complementaire;
        display_line;

        display(Format('total cotisations obligatoires %s %10.2f', [f_spaces(41), l_total]));

        display_line;
        _compute_formation_professionnelle;

        display_line;
        _compute_csg_crds;

        display_line;
        display(Format('%s %10.2f   %6.2f', [f_spaces(58+ 5), l_total, l_total/ p_salaire]));
      end; // compute_charges

    Destructor c_calcul_charges_sarl.Destroy;
      begin
        m_c_key_list.Free;
        m_c_value_list.Free;
        
        Inherited;
      end; // Destroy

    begin // u_c_calcul_charges_sarl
    end. // u_c_calcul_charges_sarl



