// 001 u_c_xxx
// 04 mar 00

// -- mainly for reading (analysis

(*$r+*)

unit u_c_stringlist;
  interface
    uses Classes, u_c_basic_object;

    type c_stringlist=
             class(c_basic_object)
               m_c_stringlist: tStringList;
               m_path, m_file_name: String;
               m_trace_stringlist: Boolean;

               // -- the index of the current line
               m_list_index: Integer;

               m_the_line, m_trimmed_line, m_lowercase_trimmed_line: String;
               m_indentation, m_length, m_trimmed_length, m_line_index: Integer;
               m_position: Integer;
               m_modified: Boolean;

               Constructor create_stringlist(p_name, p_path, p_file_name: String);

               function f_c_self: c_stringlist;
               function f_count: Integer;

               procedure reset_stringlist;
               procedure assign_string(p_string: String);
               procedure stop_if_empty;
               function f_stop_if_empty: Boolean;

               // procedure load_from_file(p_path, p_file_name: String);
               procedure read_line;
               procedure read_previous_line;
               function f_end_of_file: Boolean;
               function f_read_line: Boolean;
               procedure skip_blank_lines;
               function f_test_next_line: String;
               function f_is_blank_or_comment: Boolean;

               function f_c_unique_trimmed_list: tStringList;
               function f_find_line_containing(p_sub_string: String): Boolean;

               procedure delete_current_line;
               procedure insert_current_line(p_string: String);
               procedure replace_current_line(p_string: String);

               function f_next_value: String;
               function f_end_of_trimmed_line: String;

               function f_full_line_next_value: String;
               function f_end_of_full_line: String;

               function f_read_fixed(p_length: Integer): String;

               procedure invert_stringlist;

               function f_trimmed_end_of_the_line(p_index: integer): String;

               procedure save_to_file(p_full_file_name: String);
               procedure save_to_file_if_modified(p_full_file_name: String);
               procedure load_from_file(p_path, p_file_name: String);

               Destructor Destroy; Override;
             end; // c_stringlist

  implementation
    uses SysUtils, u_c_display
        , u_strings, u_stringlist
        ;

    // -- c_stringlist

    Constructor c_stringlist.create_stringlist(p_name, p_path, p_file_name: String);
      begin
        Inherited create_basic_object(p_name);
        m_c_stringlist:= tStringList.Create;
        m_path:= p_path; m_file_name:= p_file_name;

        load_from_file(m_path, m_file_name);
      end; // create_stringlist

    function c_stringlist.f_c_self: c_stringlist;
      begin
        Result:= Self;
      end; // f_c_self

    function c_stringlist.f_count: Integer;
      begin
        Result:= m_c_stringlist.Count;
      end; // f_count
(*
    procedure c_stringlist.load_from_file(p_path, p_file_name: String);
      begin
        m_c_stringlist.LoadFromFile(p_path+ p_file_name);
        m_path:= p_path; m_file_name:= p_file_name;
      end; // load_from_file
*)
    procedure c_stringlist.reset_stringlist;
      begin
        // -- start at - 1 so that m_list_index represents the CURRENT list index
        m_list_index:= -1;
        // -- anticipated read, to work like a file "WHILE NOT Eof DO"
        read_line;
      end; // reset_stringlist

    procedure c_stringlist.assign_string(p_string: String);
      begin
        m_c_stringlist.Text:= p_string;
        reset_stringlist;
      end; // assign_string

    function c_stringlist.f_stop_if_empty: Boolean;
      begin
        if m_c_stringlist.Count= 0
          then begin
              Result:= True;
              if not FileExists(m_path+ m_file_name)
                then display_bug_stop('nothing_in '+ m_path+ m_file_name+ ' NOT_FOUND')
                else display_bug_stop('nothing_in '+ m_path+ m_file_name);
            end
          else Result:= False;  
      end; // f_stop_if_empty

    procedure c_stringlist.stop_if_empty;
      begin
        f_stop_if_empty;
      end; // stop_if_empty

    procedure c_stringlist.read_line;
      begin
        // stop(IntToStr(m_list_index));
        Inc(m_list_index);
        with m_c_stringlist do
          if m_list_index>= Count
            then begin
                m_the_line:= '';
                m_trimmed_line:= '';
                m_lowercase_trimmed_line:= '';
                m_indentation:= 0;
                m_line_index:= 1;
              end
            else begin
                m_the_line:= Strings[m_list_index];
                m_trimmed_line:= Trim(m_the_line);
                m_lowercase_trimmed_line:= LowerCase(m_trimmed_line);
                m_indentation:= f_starting_space_count(m_the_line);
                m_length:= Length(m_the_line); m_trimmed_length:= Length(m_trimmed_line);
                m_line_index:= 1;
              end;

        // -- will not display the first line (since create an does an anticipated read)
        if m_trace_stringlist
          then display(Format('%5d ', [m_list_index])+ m_the_line+ '<');
      end; // read_line

    procedure c_stringlist.read_previous_line;
      begin
        Dec(m_list_index);
        with m_c_stringlist do
          if m_list_index< 0
            then begin
                m_the_line:= '';
                m_trimmed_line:= '';
                m_lowercase_trimmed_line:= '';
                m_indentation:= 0;
              end
            else begin
                m_the_line:= Strings[m_list_index];
                m_trimmed_line:= Trim(m_the_line);
                m_lowercase_trimmed_line:= LowerCase(m_trimmed_line);
                m_indentation:= f_starting_space_count(m_the_line);
              end;

        // -- will not display the first line (since create an does an anticipated read)
        if m_trace_stringlist
          then display(Format('%5d ', [m_list_index])+ m_the_line+ '<');
      end; // read_previous_line

    function c_stringlist.f_end_of_file: Boolean;
      begin
        Result:= m_list_index>= m_c_stringlist.Count
      end; // f_end_of_file

    function c_stringlist.f_read_line: Boolean;
        // -- to be able to write "IF Not f_read_line then Break" or IF f_read_line Then handle"
      begin
        read_line;
        Result:= not f_end_of_file;
      end; // f_read_line

    procedure c_stringlist.skip_blank_lines;
      begin
        while not f_end_of_file and (m_trimmed_line= '') do
          read_line;
      end; // skip_blank_lines

    function c_stringlist.f_test_next_line: String;
      begin
        with m_c_stringlist do
          if m_list_index+ 1< Count
            then Result:= Strings[m_list_index+ 1]
            else Result:= '';
      end; // f_test_next_line

    function c_stringlist.f_is_blank_or_comment: Boolean;
      begin
        Result:= (m_trimmed_length= 0)
            or ((m_length> 0) and (m_trimmed_line[1]= '/'))
      end; // f_is_blank_or_comment

    function c_stringlist.f_c_unique_trimmed_list: tStringList;
      begin
        Result:= tStringList.Create;
        while not f_end_of_file do
        begin
          add_unique_name(Result, Trim(m_trimmed_line));
          read_line;
        end;
      end; // f_c_unique_trimmed_list

    function c_stringlist.f_find_line_containing(p_sub_string: String): Boolean;
        // -- find from current line
        // -- CAUTION: if does not move after success, does not move
      begin
        m_position:= -1;
        Result:= False;
        while not f_end_of_file do
        begin
          m_position:= Pos(p_sub_string, m_the_line);
          if m_position> 0
            then begin
                Result:= True;
                Break;
              end;
          read_line;
        end; // while not f_end_of_line
      end; // f_find_line_containing

    procedure c_stringlist.delete_current_line;
        // -- remove a line from the stringlist
        // -- do NOT read the new current line
      begin
        m_c_stringlist.Delete(m_list_index);
        Dec(m_list_index);
        m_modified:= True;
      end; // delete_current_line

    procedure c_stringlist.insert_current_line(p_string: String);
        // -- insert a line at the current position
        // -- do NOT read the next line
      begin
        m_c_stringlist.Insert(m_list_index, p_string);
        m_modified:= True;
      end; // insert_current_line

    procedure c_stringlist.replace_current_line(p_string: String);
      begin
        m_c_stringlist[m_list_index]:= p_string;
        m_modified:= True;
      end; // replace_current_line

    function c_stringlist.f_next_value: String;
      begin
        Result:= f_string_extract_non_blank(m_trimmed_line, m_line_index);
      end; // f_next_value

    function c_stringlist.f_end_of_trimmed_line: String;
      begin
        Result:= m_trimmed_line;
        System.Delete(Result, 1, m_line_index);
        Result:= Trim(Result);
      end; // c_stringlist.

    function c_stringlist.f_full_line_next_value: String;
      begin
        Result:= f_string_extract_non_blank(m_the_line, m_line_index);
      end; // f_full_line_next_value

    function c_stringlist.f_end_of_full_line: String;
      begin
        Result:= m_the_line;
        System.Delete(Result, 1, m_line_index);
        // Result:= Trim(Result);
      end; // f_end_of_full_line

    function c_stringlist.f_read_fixed(p_length: Integer): String;
      begin
        if m_length< m_line_index+ p_length
          then Result:= '???_fixed' // ; display_bug_stop('')
          else begin
              Result:= f_extract_string_start_size(m_the_line, m_line_index, p_length);
              Inc(m_line_index, p_length);
            end;
      end; // f_read_fixed

    procedure c_stringlist.invert_stringlist;
      var l_c_stringlist, l_c_exchange_stringlist: tStringList;
          l_list_index, l_count: Integer;
      begin
        l_c_stringlist:= tStringList.Create;

        l_count:= m_c_stringlist.Count;
        for l_list_index:= 0 to l_count- 1 do
          l_c_stringlist.Add(m_c_stringlist[l_count- 1- l_list_index]);

        l_c_exchange_stringlist:= m_c_stringlist;
        m_c_stringlist:= l_c_stringlist;
        l_c_exchange_stringlist.Free;

        reset_stringlist;
(*
        display(m_c_stringlist.Text);
        stop('');
*)
      end; // invert_stringlist

    function c_stringlist.f_trimmed_end_of_the_line(p_index: integer): String;
      begin
        Result:= Trim(Copy(m_the_line, p_index, m_length));
      end; // f_trimmed_end_of_the_line

    // -- help

    procedure c_stringlist.save_to_file(p_full_file_name: String);
      begin
        m_c_stringlist.SaveToFile(p_full_file_name);
      end; // save_to_file

    procedure c_stringlist.save_to_file_if_modified(p_full_file_name: String);
      begin
        if m_modified
          then m_c_stringlist.SaveToFile(p_full_file_name);
      end; // save_to_file_if_modified

    procedure c_stringlist.load_from_file(p_path, p_file_name: String);
      begin
        if (m_file_name<> '') and FileExists(m_path+ m_file_name)
          then m_c_stringlist.LoadFromFile(m_path+ m_file_name);
        reset_stringlist;
      end; // load_from_file

    Destructor c_stringlist.Destroy;
      begin
        m_c_stringlist.Free;
        Inherited;
      end; // Destroy

    begin // u_c_stringlist
    end. // u_c_stringlist


